/*
 *  Copyright (C) 2002 Matthew Colyer (linuxcoder@colyer.org)
 *  Copyright (C) 2002 Anatoly Asviyan (aka Arsen)
 *	Copyright (C) 2000 Peter Zelezny
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "config.h"
#include "launchbar.h"
#include "support.h"

launchbaritem_data *launchbaritem;
GtkTooltips *launchbar_tips;

void
xml_element_parse_launchbar(GMarkupParseContext * context,
			    gchar * element_name, gchar ** attribute_names,
			    gchar ** attribute_values, gpointer userdata,
			    GError ** error);
void panel_launchbar_create_item(GtkWidget * box, gchar * name,
				 gchar * action, gchar * image_filename);


GtkWidget *panel_launchbar_load()
{
	gchar *buffer, *user_config_file;
	gint bytes_read;
	GError *error = NULL;
	GMarkupParser xml_configuration;
	GMarkupParseContext *xml_parser;
	launchbaritem_data *root_launchbaritem, *temp_launchbaritem;
	GtkWidget *launchbar_box;

	//Check for the user configuration file and then the global
	user_config_file =
	    g_build_filename(g_get_home_dir(), LB_CONFIG_FILE, NULL);
	
	g_file_get_contents (user_config_file, &buffer, &bytes_read, &error);
	if (error) {
		//No user launchbar file found
		g_printerr
			("No User launchbar configuration loaded: %s\n",
			 error->message);
		g_error_free(error);
		error = NULL;

		//Attempt to load a global configuration file
		g_file_get_contents (LB_GLOBAL_CONFIG_FILE, &buffer, &bytes_read, &error);

		if (error) {
			//No menu file found
			g_printerr
				("No Global launchbar configuration loaded: %s\n",
				 error->message);
			g_error_free(error);
			return NULL;
		}
	}
	g_free(user_config_file);

	//Setup the xml parser
	xml_configuration.start_element = (void *) &xml_element_parse_launchbar;
	xml_configuration.end_element = NULL;
	xml_configuration.text = NULL;
	xml_configuration.passthrough = NULL;
	xml_configuration.error = NULL;

	//Create the parser
	xml_parser =
	    g_markup_parse_context_new(&xml_configuration, 0, NULL, NULL);

	//Connect the first lauchbar item to the local variable
	root_launchbaritem = (launchbaritem_data *) g_malloc(sizeof(launchbaritem_data));
	launchbaritem = root_launchbaritem;
	
	//Parse the buffer
	g_markup_parse_context_parse(xml_parser, buffer, bytes_read, NULL);
	
	g_markup_parse_context_free (xml_parser);
	g_free(buffer);

	//Create a container to hold the widgets
	launchbar_box = gtk_hbox_new(TRUE, 1);
	gtk_widget_show(launchbar_box);
	
	//Create the tooltips structure to hold the tooltips
	
	launchbar_tips = gtk_tooltips_new();
	
	launchbaritem = root_launchbaritem;
	while (launchbaritem->next) {
		panel_launchbar_create_item(launchbar_box,
					    launchbaritem->name,
					    launchbaritem->action,
					    launchbaritem->image_filename);
		/* You can not free launchbaritem->name because it is used when
		   displaying tooltips */
		/* You can not free launchbaritem->action because it is used when
		   launching applications */
		g_free(launchbaritem->image_filename);
		temp_launchbaritem = launchbaritem;
		launchbaritem = (launchbaritem_data *) launchbaritem->next;
		free(temp_launchbaritem);
	}

	
	return launchbar_box;
}

void
xml_element_parse_launchbar(GMarkupParseContext * context,
			    gchar * element_name, gchar ** attribute_names,
			    gchar ** attribute_values, gpointer user_data,
			    GError ** error)
{
	gint a;

	if (strcmp(element_name, "launchbaritem") == 0) {
		for (a = 0; a < 3; a++) {
			if (strcmp(attribute_names[a], "name") == 0)
				launchbaritem->name =
				    g_strdup(attribute_values[a]);
			if (strcmp(attribute_names[a], "action") == 0)
				launchbaritem->action =
				    g_strdup(attribute_values[a]);
			if (strcmp(attribute_names[a], "image") == 0)
				launchbaritem->image_filename =
				    g_strdup(attribute_values[a]);
		}
		launchbaritem->next =
		    (launchbaritem_data *)
		g_malloc(sizeof(launchbaritem_data));
		launchbaritem = (launchbaritem_data *) launchbaritem->next;
		launchbaritem->next = NULL;
		return;
	}
	if (strcmp(element_name, "launchbarseperator") == 0) {
		launchbaritem->name = g_strdup("-");
		launchbaritem->action = NULL;
		launchbaritem->image_filename = NULL;
		launchbaritem->next =
		    (launchbaritem_data *)
		    g_malloc(sizeof(launchbaritem_data));
		launchbaritem = (launchbaritem_data *) launchbaritem->next;
		launchbaritem->next = NULL;
		return;
	}
	//Execution should never reach this point
	g_printerr
	    ("Launchbar configuration syntax error!\n Check the syntax of your launchbar configuration file.\n");
}

void
panel_launchbar_create_item(GtkWidget * box, gchar * name, gchar * action,
			    gchar * image_filename)
{
	GtkWidget *image, *button, *seperator;
	GError *err = NULL;
	gchar *failed_icon_image;
	GdkPixbuf *icon, *icon_scaled;
	
	
	if (strcmp(name, "-") == 0) {
		seperator = gtk_vseparator_new();
		gtk_box_pack_start(GTK_BOX(box), seperator, FALSE, FALSE,
				   0);
		return;
	}
	icon = gdk_pixbuf_new_from_file(image_filename, &err);
	if (err) {
		g_printerr("Launchbar icon not loaded:\n %s\n",
			   err->message);
		g_error_free(err);
		err = NULL;

		failed_icon_image =
		    g_build_filename(PKGIMAGEDIR, "not-found.png", NULL);
		icon = gdk_pixbuf_new_from_file(failed_icon_image, &err);
		g_free(failed_icon_image);

		if (err) {
			//Critical Error, program will exit
			g_error("%s\n", err->message);
		}
	}
	icon_scaled =
	    gdk_pixbuf_scale_simple(icon, ICONSIZE, ICONSIZE,
				    GDK_INTERP_BILINEAR);

	image = gtk_image_new_from_pixbuf(icon_scaled);

	gdk_pixbuf_unref(icon);
	gdk_pixbuf_unref(icon_scaled);

	button = gtk_button_new();

	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
	gtk_container_set_border_width(GTK_CONTAINER(button), 0);

	gtk_container_add(GTK_CONTAINER(button), image);
	
	g_signal_connect_swapped(GTK_OBJECT(button), "clicked",
			   G_CALLBACK(execute_program), action);
	
	gtk_widget_show(button);
	
	gtk_tooltips_set_tip (GTK_TOOLTIPS (launchbar_tips), button,
				 name,
				 name);
	
	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
}
