/*
 *  Copyright (C) 2002 Matthew Colyer (linuxcoder@colyer.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <gtk/gtk.h>

#include "config.h"
#include "menu.h"
#include "support.h"

void
xml_element_parse(GMarkupParseContext * context, gchar * element_name,
		  gchar ** attribute_names, gchar ** attribute_values,
		  gpointer userdata, GError ** error);

menuitem_data *menuitem;

GtkWidget *panel_menu_load()
{
	GtkWidget *menu;
	gchar *buffer, *user_config_file;
	gint bytes_read;
	GError *err = NULL;
	GMarkupParser xml_configuration;
	GMarkupParseContext *xml_parser;
	menuitem_data *root_menuitem, *temp_menuitem;

	//Check for the user configuration file and then the global
	user_config_file =
	    g_build_filename(g_get_home_dir(), MENU_CONFIG_FILE, NULL);

	g_file_get_contents(user_config_file, &buffer, &bytes_read, &err);
	if (err) {
		//No menu file found
		g_printerr("No User menu configuration loaded: %s\n",
			   err->message);
		g_error_free(err);
		err = NULL;

		g_free(buffer);

		//Attempt to load a Global configuration file
		g_file_get_contents(MENU_GLOBAL_CONFIG_FILE, &buffer,
				    &bytes_read, &err);

		if (err) {
			//No menu file found
			g_printerr
			    ("No Global menu configuration loaded: %s\n",
			     err->message);
			g_error_free(err);
			return NULL;
		}
	}
	g_free(user_config_file);

	//Setup the xml parser
	xml_configuration.start_element = (void *) &xml_element_parse;
	xml_configuration.end_element = NULL;
	xml_configuration.text = NULL;
	xml_configuration.passthrough = NULL;
	xml_configuration.error = NULL;

	//Create the parser
	xml_parser =
	    g_markup_parse_context_new(&xml_configuration, 0, NULL, NULL);

	//Connect the first menu item to the local variable
	root_menuitem = (menuitem_data *) g_malloc(sizeof(menuitem_data));
	menuitem = root_menuitem;
	
	//Parse the buffer
	g_markup_parse_context_parse(xml_parser, buffer, bytes_read, NULL);
	
	g_markup_parse_context_free (xml_parser);
	g_free(buffer);

	//Add to menu
	menu = gtk_menu_new();

	
	menuitem = root_menuitem;
	while (menuitem->next) {
		panel_menu_create_item(menu, menuitem->name,
				       menuitem->action,
				       menuitem->image_filename);
		g_free(menuitem->name);
		/* You can not free menuitem->action because it is used when
		   launching applications */
		g_free(menuitem->image_filename);
		temp_menuitem = menuitem;
		menuitem = (menuitem_data *) menuitem->next;
		free(temp_menuitem);
	}
	

	return menu;
}

void
xml_element_parse(GMarkupParseContext * context, gchar * element_name,
		  gchar ** attribute_names, gchar ** attribute_values,
		  gpointer user_data, GError ** error)
{
	gint a;

	if (strcmp(element_name, "menuitem") == 0) {
		for (a = 0; a < 3; a++) {
			if (strcmp(attribute_names[a], "name") == 0)
				menuitem->name =
				    g_strdup(attribute_values[a]);
			if (strcmp(attribute_names[a], "action") == 0)
				menuitem->action =
				    g_strdup(attribute_values[a]);
			if (strcmp(attribute_names[a], "image") == 0)
				menuitem->image_filename =
				    g_strdup(attribute_values[a]);
		}
		menuitem->next =
		    (menuitem_data *) g_malloc(sizeof(menuitem_data));
		menuitem = (menuitem_data *) menuitem->next;
		menuitem->next = NULL;
		return;
	}
	if (strcmp(element_name, "menuseperator") == 0) {
		menuitem->name = g_strdup("-");
		menuitem->action = NULL;
		menuitem->image_filename = NULL;
		menuitem->next =
		    (menuitem_data *) g_malloc(sizeof(menuitem_data));
		menuitem = (menuitem_data *) menuitem->next;
		menuitem->next = NULL;
		return;
	}
	//Execution should never reach this point
	g_printerr
	    ("Menu configuration syntax error!\n Check the syntax of your menu configuration file.\n");
}


void
panel_menu_create_item(GtkWidget * menu, gchar * name, gchar * action,
		       gchar * image_filename)
{
	GtkWidget *menu_item, *image;

	if (strcmp(name, "-") == 0) {
		menu_item = gtk_separator_menu_item_new();
		gtk_menu_append(GTK_MENU(menu), menu_item);
		gtk_widget_show(menu_item);
		return;
	}
	menu_item = gtk_image_menu_item_new_with_label(name);
	image = gtk_image_new_from_file(image_filename);
	gtk_widget_show(image);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item),
				      image);

	gtk_menu_append(GTK_MENU(menu), menu_item);
	g_signal_connect_swapped(G_OBJECT(menu_item), "activate",
				 G_CALLBACK(execute_program), action);
	gtk_widget_show(menu_item);
}

void
panel_menu_create_submenu(GtkWidget * menu, gchar * name, gchar * image)
{

}
